Package: fcaR
Title: Formal Concept Analysis
Version: 2.1.0
Authors@R: 
    c(person(given = "Domingo",
             family = "Lopez Rodriguez",
             role = c("aut", "cre"),
             email = "dominlopez78@gmail.com",
             comment = c(ORCID = "0000-0002-0172-1585")),
      person(given = "Angel",
             family = "Mora",
             role = "aut",
             email = "amorabonilla@gmail.com"))
Description: Provides tools to perform fuzzy formal concept analysis, presented in Wille (1982) <doi:10.1007/978-3-642-01815-2_23> and in Ganter and Obiedkov (2016) <doi:10.1007/978-3-662-49291-8>.  It provides functions to load and save a formal context, extract its concept lattice and implications.  In addition, one can use the implications to compute semantic closures of fuzzy sets and, thus, build recommendation systems. Boolean Matrix Factorization (BMF) is provided by several algorithms (such as GreConD, ASSO, RSF, RSF-ES, GreEss, PaNDa+, and Hyper+).
License: GPL-3
URL: https://github.com/Malaga-FCA-group/fcaR,
        https://neuroimaginador.github.io/fcaR/
BugReports: https://github.com/Malaga-FCA-group/fcaR/issues
Depends: R (>= 4.1)
Imports: dplyr, glue, Matrix, methods, R6, rlang, Rcpp, registry,
        settings, shiny, stringr, tibble, tidyr, tidyselect, purrr, cli
Suggests: bslib, DT, ggplot2, ggraph, grDevices, igraph, plotly,
        shinyalert, shinyjs, shinyWidgets, visNetwork, future,
        promises, rhandsontable, arules, covr, fractional, jsonlite,
        knitr, markdown, miniUI, parallel, rmarkdown, rstudioapi,
        testthat (>= 2.1.0), tictoc, tikzDevice, tinytex, yaml
LinkingTo: Rcpp, BH
VignetteBuilder: knitr
Encoding: UTF-8
LazyData: true
RoxygenNote: 7.3.3
NeedsCompilation: yes
Packaged: 2026-07-27 10:34:57 UTC; domingo
Author: Domingo Lopez Rodriguez [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-0172-1585>),
  Angel Mora [aut]
Maintainer: Domingo Lopez Rodriguez <dominlopez78@gmail.com>
Repository: CRAN
Date/Publication: 2026-07-27 11:40:14 UTC
Built: R 4.6.0; x86_64-apple-darwin20; 2026-07-27 15:16:22 UTC; unix
Archs: fcaR.so.dSYM
