# Generated by roxygen2: do not edit by hand

S3method("[",ConceptSet)
S3method("[",FormalContext)
S3method("[",ImplicationSet)
S3method("[",RuleSet)
S3method("[",Set)
S3method(arrange,FormalContext)
S3method(arrange,RuleSet)
S3method(as.list,ConceptSet)
S3method(filter,FormalContext)
S3method(filter,RuleSet)
S3method(length,ConceptSet)
S3method(mutate,FormalContext)
S3method(mutate,RuleSet)
S3method(print,tikz_code)
S3method(rename,FormalContext)
S3method(save_tikz,tikz_code)
S3method(select,FormalContext)
S3method(slice,RuleSet)
export("%&%")
export("%-%")
export("%<=%")
export("%==%")
export("%entails%")
export("%holds_in%")
export("%respects%")
export("%|%")
export("%~%")
export(BondLattice)
export(Concept)
export(ConceptLattice)
export(ConceptSet)
export(FormalContext)
export(ImplicationSet)
export(RandomContext)
export(RandomDistributiveContext)
export(RuleSet)
export(Set)
export(as_Set)
export(as_vector)
export(attribute_set)
export(bonds)
export(calculate_grades)
export(calculate_separation)
export(conceptRegistry)
export(context_from_json)
export(equivalencesRegistry)
export(fcaR_options)
export(fetch_context)
export(find_causal_rules)
export(get_fcarepository_contexts)
export(implications_from_json)
export(is_bond)
export(iterative_recommender)
export(lattice_from_json)
export(lattice_plot)
export(object_set)
export(parse_implications)
export(print_repo_details)
export(randomize_context)
export(recommendation_table)
export(rules_from_json)
export(run_fcaRviz)
export(save_tikz)
export(scalingRegistry)
export(select_repository_context)
import(R6)
import(registry)
importClassesFrom(Matrix,CsparseMatrix)
importFrom(Matrix,Matrix)
importFrom(Matrix,colSums)
importFrom(Matrix,rowSums)
importFrom(Matrix,summary)
importFrom(Matrix,t)
importFrom(Rcpp,sourceCpp)
importFrom(cli,cli_alert_info)
importFrom(dplyr,arrange)
importFrom(dplyr,filter)
importFrom(dplyr,mutate)
importFrom(dplyr,rename)
importFrom(dplyr,select)
importFrom(dplyr,slice)
importFrom(glue,glue)
importFrom(methods,.hasSlot)
importFrom(methods,as)
importFrom(methods,cbind2)
importFrom(methods,is)
importFrom(methods,new)
importFrom(methods,slotNames)
importFrom(purrr,map)
importFrom(purrr,pluck)
importFrom(rlang,.data)
importFrom(rlang,enquo)
importFrom(rlang,enquos)
importFrom(rlang,eval_tidy)
importFrom(rlang,expr)
importFrom(rlang,new_data_mask)
importFrom(shiny,runApp)
importFrom(stats,na.omit)
importFrom(stats,qnorm)
importFrom(stats,rgamma)
importFrom(stats,setNames)
importFrom(stringr,fixed)
importFrom(stringr,str_replace_all)
importFrom(stringr,str_split)
importFrom(stringr,str_squish)
importFrom(stringr,str_which)
importFrom(stringr,str_wrap)
importFrom(tidyselect,eval_rename)
importFrom(tidyselect,eval_select)
importFrom(utils,download.file)
useDynLib(fcaR, .registration = TRUE)
